/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.dept.impl;

import cn.hutool.core.util.ObjectUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.excel.dept.DepartmentExportExcel;
import com.yiring.app.param.dept.DepartmentAddParam;
import com.yiring.app.param.dept.DepartmentFindParam;
import com.yiring.app.service.dept.DepartmentService;
import com.yiring.app.service.zy.ZyHttpService;
import com.yiring.app.vo.dept.DepartmentInfoVo;
import com.yiring.app.vo.dept.DepartmentVo;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.dept.DepartmentRepository;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 部门管理
 * @author LJ-2204
 * @date 2022/4/19
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class DepartmentServiceImpl implements DepartmentService {

    @Resource
    DepartmentRepository departmentRepository;

    @Resource
    ZyHttpService zyHttpService;

    // todo
    @Override
    public Result<String> addDepartment(DepartmentAddParam param) {
        long pidExist = departmentRepository.count(Example.of(Department.builder().id(param.getPid()).build()));
        if (pidExist == 0) return Result.no(Status.BAD_REQUEST, "上级部门不存在");

        long exist = departmentRepository.count(
            Example.of(Department.builder().name(param.getName()).pid(param.getPid()).build())
        );
        if (exist > 0) return Result.no(Status.BAD_REQUEST, "同级目录下，已存在该部门");

        Department department = param.transform();
        departmentRepository.save(department);
        return Result.ok();
    }

    @Override
    public Result<PageVo<DepartmentVo>> findDepartmentTree(DepartmentFindParam param) {
        Department department = Department.builder().enable(param.getEnable()).build();
        List<Department> departments = departmentRepository.findAll(Example.of(department));
        List<DepartmentVo> departmentVos = DepartmentVo.transforms(departments);
        List<DepartmentVo> departmentVoList = getChildrenList(0L, null, departmentVos);
        PageVo<DepartmentVo> pageVo = PageVo.build(departmentVoList, departmentVoList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<DepartmentInfoVo> findDepartmentInfo(IdParam idParam) {
        Optional<Department> department = departmentRepository.findOne(
            Example.of(Department.builder().id(idParam.getId()).build())
        );
        if (department.isEmpty()) return Result.no(Status.BAD_REQUEST);
        String pName = null;
        if (!ObjectUtil.equals(department.get().getPid(), 0L)) {
            Optional<Department> department_pid = departmentRepository.findOne(
                Example.of(Department.builder().id(department.get().getPid()).build())
            );
            if (department_pid.isEmpty()) return Result.no(Status.BAD_REQUEST);
            pName = department_pid.get().getName();
        }
        DepartmentInfoVo departmentInfoVo = DepartmentInfoVo.transform(department.get());
        departmentInfoVo.setPName(pName);
        return Result.ok(departmentInfoVo);
    }

    @Override
    public void exportDepartment(DepartmentFindParam param, HttpServletResponse response) {
        List<Department> departments = departmentRepository.findAll(
            Example.of(Department.builder().enable(param.getEnable()).build())
        );
        List<DepartmentExportExcel> departmentExportExcels = DepartmentExportExcel.transforms(departments);
        try (
            DefaultStreamExcelBuilder<DepartmentExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(DepartmentExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(departmentExportExcels);

            String fileName = URLEncoder.encode("部门信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出部门信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> deleteDepartment(IdParam idParam) {
        departmentRepository.deleteById(idParam.getId());
        return Result.ok();
    }

    @Override
    public Result<String> stateDepartment(IdParam idParam) {
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(idParam.getId()).build())
        );
        if (departmentOptional.isEmpty()) return Result.no(Status.BAD_REQUEST);

        Department department = departmentOptional.get();
        department.setEnable(!department.getEnable());

        departmentRepository.save(department);
        return Result.ok();
    }

    public List<DepartmentVo> getChildrenList(Long pid, String pName, List<DepartmentVo> departmentVos) {
        return departmentVos
            .stream()
            .filter(departmentVo -> pid.equals(departmentVo.getPid()))
            .peek(departmentVo -> {
                departmentVo.setPName(pName);
                departmentVo.setChildList(getChildrenList(departmentVo.getId(), departmentVo.getName(), departmentVos));
            })
            .collect(Collectors.toList());
    }
}
